%{

#include <iostream>
#include <fstream>
#include <string>
#include <vector>

#include <parser/Parser.h>
#include <parser/Part.h>

#include <autolog/autoLog.h>
#include <syntax2automata/SyntaxHelpers.h>


#include <common/comm.h>



#ifdef AUTO_DEBUG
	#define YYDEBUG 1
	#define YYERROR_VERBOSE 1	
#endif
	

common::SyntaxTreeNode* Root;
parser::ParserContext* context;

template<class T>
std::vector<T> operator+(const std::vector<T>& lhs, const std::vector<T>& rhs)
{
    std::vector<T> result(lhs);
    if (rhs.size() > 0) {
        result.insert(result.end(), rhs.begin(), rhs.end());
    }
    return result;
}

#undef YYSTYPE
#define YYSTYPE parser::Part


std::vector<common::Function> functionVector;

int yylex(void);
int yyparse(void);
extern "C" void yyerror(char* a)
{
	Log::getInstance()->logMessage(Log::MSG_ERROR, "Parser", a);
}

extern FILE* yyin;

extern "C" int yywrap()
{
	return 1;
}
%}
%expect 1 //    / -   .

%token IFF

%token IDENTIFIER              STRINGliteral
%token FLOATINGconstant        INTEGERconstant        CHARACTERconstant
%token OCTALconstant           HEXconstant			BOOLconstant

%token EXPRESSION
/*Keywords*/

%token AUTO            DOUBLE          INT             STRUCT			BOOL
%token BREAK           ELSE            LONG            SWITCH			ELLIPSIS
%token CASE            ENUM            REGISTER        TYPEDEF
%token CHAR            EXTERN          RETURN          UNION
%token CONST           FLOAT           SHORT           UNSIGNED
%token CONTINUE        FOR             SIGNED          VOID
%token DEFAULT         GOTO            SIZEOF          VOLATILE
%token DO              IF              STATIC          WHILE


/* Multi-Character operators */
%token  ARROW            /*    ->                              */
%token  ICR DECR         /*    ++      --                      */
%token  LS RS            /*    <<      >>                      */
%token  LE GE EQ NE      /*    <=      >=      ==      !=      */
%token  ANDAND OROR      /*    &&      ||                      */


/* modifying assignment operators */
%token MULTassign  DIVassign    MODassign   /*   *=      /=      %=      */
%token PLUSassign  MINUSassign              /*   +=      -=              */
%token LSassign    RSassign                 /*   <<=     >>=             */
%token ANDassign   ERassign     ORassign    /*   &=      ^=      |=      */
%%

input: translation_unit
	{
		Root=$1.link;
		//globalVariables = variables;
	}
	;

translation_unit:
        external_definition
        {$$.link = $1.link;}
        | translation_unit external_definition
      	{
      		$$.link = new common::SyntaxTreeNode(common::SyntaxTreeNode::NTYPE_SEMICOLON,"");
      		$$.link->_left = $1.link;
      		$$.link->_right = $2.link;
      	}
		;
		
external_definition:
        function_definition
        {
			$$.link = $1.link;
        }
        | declaration
        {
			
			$$.link = $1.link;
		
        }
        ;
        
function_definition: 
		
		type_specifier identifier_declarator compound_statement
		{
			int bracketPos = $2.text.find("(");
			
			std::string name = $2.text.substr(0,bracketPos);
			s2a::removeDataPrefix(name);
			std::string cleanedDefinition = $1.text + " " + $2.text + " " + $3.text;
			s2a::removeDataPrefix(cleanedDefinition);
			
			$$.link = new common::SyntaxTreeNode(common::SyntaxTreeNode::NTYPE_PROCEDURE, cleanedDefinition, context->lineNumber, $1.lineNumber, $3.lineNumber); //TODO: do something with name
			$$.link->_left = $3.link;
			common::Function* func = new common::Function($1.text + " " + name);
			func->localVariables = context->variables;
			context->knownFunctions.push_back(func);
			context->variables.clear();
		}
		;
				
			 


statement: compound_statement
			{$$.link = $1.link;}
		| selection_statement
		{$$.link = $1.link;}
		 | iteration_statement
		  {$$.link = $1.link;}
		|expression_statement
		{$$.link = $1.link;}
		
		;
compound_statement:
			'{''}'
			{$$.link = NULL; $$.lineNumber = $2.lineNumber; }
			|'{' statement_list '}'
			{$$.link = $2.link;$$.lineNumber = $3.lineNumber;}
			|'{' declaration_list statement_list '}'
			{
				$$.link = new common::SyntaxTreeNode(common::SyntaxTreeNode::NTYPE_SEMICOLON,"",context->lineNumber);	
				$$.link->_left = $2.link;
				$$.link->_right = $3.link;
				$$.lineNumber = $2.lineNumber;
			}
			|jump_statement
			{$$.link = $1.link;$$.text = "";$$.lineNumber = $1.lineNumber;}
			;
			
jump_statement:
   
        RETURN comma_expression_opt ';'
        {
			$$.link = new common::SyntaxTreeNode(common::SyntaxTreeNode::NTYPE_RETURN,$2.text,context->lineNumber);
			$$.link->dependsOn = $2.dependsOn;
			$$.dependsOn = $2.dependsOn;
			$$.text = "";
        }
        ;

statement_list:
        statement
        {$$.link = $1.link; $$.dependsOn = $1.dependsOn}
        | statement_list statement
        {
			std::string a = "";
			$$.link = new common::SyntaxTreeNode(common::SyntaxTreeNode::NTYPE_SEMICOLON,a,context->lineNumber);
			
			$$.link->_left = $1.link;
			$$.link->_right=$2.link;
        }
        ;
expression_statement:
 comma_expression_opt ';'
 {$$.text = $1.text;$$.link = $1.link; $$.dependsOn = $1.dependsOn;}
        ;
      
   
selection_statement:
          	IFF '(' comma_expression ')' statement ELSE statement
			{
				$$.link = new common::SyntaxTreeNode(common::SyntaxTreeNode::NTYPE_IF, $3.text, $1.lineNumber, $5.lineNumber, $7.lineNumber);
				$$.link->_left = $5.link;
				$$.dependsOn = $$.link->dependsOn = $3.dependsOn;
				$$.link->_right = $7.link;
			}
			| IFF '(' comma_expression ')' statement
			{
			
				$$.link = new common::SyntaxTreeNode(common::SyntaxTreeNode::NTYPE_IF, $3.text, $1.lineNumber, $1.lineNumber, $5.lineNumber);
				$$.dependsOn = $$.link->dependsOn = $3.dependsOn;
				$$.link->_left = $5.link;
				
			}
        ;
	
iteration_statement:
        WHILE '(' comma_expression ')' statement
       {
		        $$.link = new common::SyntaxTreeNode(common::SyntaxTreeNode::NTYPE_WHILE, $3.text, $1.lineNumber, $5.lineNumber);
		        $$.link->dependsOn = $3.dependsOn;
				$$.link->_left = $5.link;
				
       }
     /*   | DO statement WHILE '(' comma_expression ')' ';'*/
      /*  | FOR '(' comma_expression_opt ';' comma_expression_opt ';'
                comma_expression_opt ')' statement*/
        ;
        
/*////////////////////Expressions///////////////////////////////*/
constant:
        INTEGERconstant
        {
				$$.text = $1.text; $$.link = $1.link;
		}
        | FLOATINGconstant
        {
				$$.text = $1.text; $$.link = $1.link;
		}
        | OCTALconstant
        {
				$$.text = $1.text; $$.link = $1.link;
		}
        | HEXconstant
        {
				$$.text = $1.text; $$.link = $1.link;
		}
		| BOOLconstant
		{
				$$.text = $1.text; $$.link = $1.link;
		}
		|CHARACTERconstant
		{
				$$.text = $1.text; $$.link = $1.link;
		}
        ;


primary_expression:
        IDENTIFIER
        {
				$$.text = "data->"+$1.text; $$.link = $1.link;
				$$.dependsOn.push_back($$.text);
		}
        | constant
        {
				$$.text = $1.text; $$.link = $1.link;
		}
       
        ;	
 postfix_expression:
        primary_expression
         {
				$$.text = $1.text; $$.link = $1.link;
				$$.dependsOn = $1.dependsOn;
		}
		| postfix_expression '('')' //function call without parameters
		{
		//   
			
			s2a::removeDataPrefix($1.text);
			
			
			common::SyntaxTreeNode::NodeType type = common::SyntaxTreeNode::NTYPE_UNKNOWN_FUNCTION_CALL;
			for(unsigned int i = 0; i < context->knownFunctions.size(); i++) {
				if(context->knownFunctions[i]->name == $1.text) {
					type = common::SyntaxTreeNode::NTYPE_KNOWN_FUNCTION_CALL; 
				}
		    }
		
			  if (type == common::SyntaxTreeNode::NTYPE_KNOWN_FUNCTION_CALL) {
		        $$.link = new common::SyntaxTreeNode(type,$1.text + "Automaton()" ,context->lineNumber);
		        //functionVector.push_back(common::Function($1.text + "()"));//TODO: later
		    } else {
		        $$.link = new common::SyntaxTreeNode(type,$1.text + "()",context->lineNumber);
		    }
		}
		| postfix_expression '(' argument_expression_list ')'
		{
		    s2a::removeDataPrefix($1.text);
		    common::SyntaxTreeNode::NodeType type = common::SyntaxTreeNode::NTYPE_UNKNOWN_FUNCTION_CALL;
			for(unsigned int i = 0; i < context->knownFunctions.size(); i++) {
				if(context->knownFunctions[i]->name == $1.text) {
					type = common::SyntaxTreeNode::NTYPE_KNOWN_FUNCTION_CALL; 
									
				}
		    }
		    if (type == common::SyntaxTreeNode::NTYPE_KNOWN_FUNCTION_CALL) {
		        $$.link = new common::SyntaxTreeNode(type,$1.text + "Automaton(" + $3.text + ')' ,context->lineNumber);
		        //functionVector.push_back(common::Function($1.text + " (" + $3.text + ")")); TODO:Later
		    } else {
		        $$.link = new common::SyntaxTreeNode(type,$1.text + "(" + $3.text + ')' ,context->lineNumber);
		    }
		    $$.link->dependsOn = $3.dependsOn;
			
		
		}
        | postfix_expression '[' comma_expression ']'
         {
				$$.text = $1.text+'['+$3.text+']'; $$.link = $1.link;
				$$.dependsOn = $1.dependsOn + $2.dependsOn;
				
		}
        | postfix_expression ICR
         {
				$$.text = $1.text+"++"; 
				$$.link = new common::SyntaxTreeNode(common::SyntaxTreeNode::NTYPE_OPERATOR,$1.text+"++",context->lineNumber);
				$$.link->_nameOfVariableToSave = $1.text;
				$$.link->dependsOn = $1.dependsOn;
				$$.link->changeSet.push_back($1.text);
		}
        | postfix_expression DECR
         {
				$$.text = $1.text+"--";
				$$.link = $$.link = new common::SyntaxTreeNode(common::SyntaxTreeNode::NTYPE_OPERATOR,$1.text+"--",context->lineNumber);
				$$.link->_nameOfVariableToSave = $1.text;
				$$.link->dependsOn = $1.dependsOn;
				$$.link->changeSet.push_back($1.text);
		}
 
argument_expression_list: ///  -   
        assignment_expression
        {
            $$.link = NULL; 
            $$.text = $1.text;
            s2a::removeDataPrefix($1.text);
            //common::Variable var($1.text);
            //$$.dependsOn.push_back(var.getName());
			$$.dependsOn.push_back($1.text); //TODO: Expression
  
        }
        | argument_expression_list ',' assignment_expression
        {
            $$.link = NULL; 
            $$.text = $1.text + ',' + $3.text;
            
			s2a::removeDataPrefix($3.text);
            //common::Variable var($3.text);
            $$.dependsOn = $1.dependsOn;
            //$$.dependsOn.push_back(var.getName());
			$$.dependsOn.push_back($3.text);             
        }
        ;
        
        unary_operator:
        '&'
        {
			$$.link = $1.link;
			$$.text = $1.text;
        }
        | '*'
        {
			$$.link = $1.link;
			$$.text = $1.text;
        }
        | '+'
        {
			$$.link = $1.link;
			$$.text = $1.text;
        }
        | '-'
        {
			$$.link = $1.link;
			$$.text = $1.text;
        }
        | '~'
        {
			$$.link = $1.link;
			$$.text = $1.text;
        }
        | '!'
        {
			$$.link = $1.link;
			$$.text = $1.text;
        }
        ;


 
unary_expression:
        postfix_expression
        {
			$$.link = $1.link;
			$$.text = $1.text;
			$$.dependsOn = $1.dependsOn;
        }
        | ICR unary_expression //add changeSets
        {
			$$.link = new common::SyntaxTreeNode(common::SyntaxTreeNode::NTYPE_OPERATOR,"++"+$2.text,context->lineNumber);
			$$.link->dependsOn = $2.dependsOn;
			$$.link->changeSet.push_back($2.text);
			$$.text = $1.text+$2.text;
			$$.link->_nameOfVariableToSave = $2.text;
			
			$$.dependsOn = $2.dependsOn;
        }
        | DECR unary_expression
        {
			$$.link = new common::SyntaxTreeNode(common::SyntaxTreeNode::NTYPE_OPERATOR,"--"+$2.text,context->lineNumber);
			$$.link->dependsOn = $2.dependsOn;
			$$.link->changeSet.push_back($2.text);
			$$.text = $1.text+$2.text;
			$$.link->_nameOfVariableToSave = $2.text;
			
			$$.dependsOn = $2.dependsOn;
        }
        | unary_operator unary_expression
        {
            $$.link = new common::SyntaxTreeNode(common::SyntaxTreeNode::NTYPE_OPERATOR,$1.text + $2.text,context->lineNumber);
            $$.link->dependsOn = $2.dependsOn;
			$$.text = $1.text + $2.text;
			$$.link->_nameOfVariableToSave = $2.text;
			$$.dependsOn = $2.dependsOn;
        }
        ;       
        
assignment_expression:
		unary_expression assignment_operator assignment_expression
        {
								
				std::string a = $1.text+' '+$2.text+' '+$3.text;
				
				if (functionVector.size() > 0) { //  assignment_expression   ,  x = 6 + f();
				
				    
				    common::SyntaxTreeNode* pNode = $$.link;
    				
				    for (unsigned int i = 0; i < functionVector.size(); i++) {
				        pNode = new common::SyntaxTreeNode(common::SyntaxTreeNode::NTYPE_SEMICOLON,"");
    				    pNode->_right =  new common::SyntaxTreeNode(common::SyntaxTreeNode::NTYPE_KNOWN_FUNCTION_CALL,functionVector[i].name + "Automaton()" ,context->lineNumber);   
                        pNode = pNode->_left;				    
				    }
    				
				    functionVector.clear();
											
				}
				$$.link = new common::SyntaxTreeNode (common::SyntaxTreeNode::NTYPE_OPERATOR,a,context->lineNumber);
								
				$$.link->dependsOn = $$.dependsOn = $1.dependsOn + $3.dependsOn;
				
				$$.link->changeSet.push_back($1.text);
				
				$$.link->_nameOfVariableToSave = $1.text;
			

			}
			
			|conditional_expression
			{
				$$.text = $1.text; $$.link = $1.link; $$.dependsOn = $1.dependsOn;
			}
			
        ;

assignment_operator:
        '='
        {$$.text = $1.text;}
        | MULTassign
        {$$.text = $1.text;}
        | DIVassign
        {$$.text = $1.text;}
        | MODassign
        {$$.text = $1.text;}
        | PLUSassign
        {$$.text = $1.text;}
        | MINUSassign
        {$$.text = $1.text;}
        | LSassign
        {$$.text = $1.text;}
        | RSassign
        {$$.text = $1.text;}
        | ANDassign
        {$$.text = $1.text;}
        | ERassign
        {$$.text = $1.text;}
        | ORassign
        {$$.text = $1.text;}
        ;
 multiplicative_expression:
        unary_expression
        {$$.text = $1.text;$$.dependsOn = $1.dependsOn;}
        | multiplicative_expression '*' primary_expression
        {$$.text = $1.text+$2.text+$3.text;$$.dependsOn = $1.dependsOn + $3.dependsOn;}
        | multiplicative_expression '/' primary_expression
        {$$.text = $1.text+$2.text+$3.text;$$.dependsOn = $1.dependsOn + $3.dependsOn;}
        | multiplicative_expression '%' primary_expression
        {$$.text = $1.text+$2.text+$3.text;$$.dependsOn = $1.dependsOn + $3.dependsOn;}
        ;

additive_expression:
        multiplicative_expression
        {$$.text = $1.text;$$.dependsOn = $1.dependsOn}
        | additive_expression '+' multiplicative_expression
        {$$.text = $1.text+$2.text+$3.text;$$.dependsOn = $1.dependsOn + $3.dependsOn;}
        | additive_expression '-' multiplicative_expression
        {$$.text = $1.text+$2.text+$3.text;$$.dependsOn = $1.dependsOn + $3.dependsOn;}
        | EXPRESSION
        {$$.text = $1.text;$$.dependsOn = $1.dependsOn}
        ;

shift_expression:
        additive_expression
          {$$.link = $1.link;$$.text = $1.text;$$.dependsOn = $1.dependsOn ;}
        | shift_expression LS additive_expression
         {$$.link = $1.link;$$.text = $1.text + $2.text+$3.text;$$.dependsOn = $1.dependsOn + $3.dependsOn;}
        | shift_expression RS additive_expression
         {$$.link = $1.link;$$.text = $1.text + $2.text+$3.text;$$.dependsOn = $1.dependsOn + $3.dependsOn;}
        ;

relational_expression:
  
        shift_expression
          {$$.link = $1.link;$$.text = $1.text;$$.dependsOn = $1.dependsOn;}
        | relational_expression '<' shift_expression
         {$$.link = $1.link;$$.text = $1.text + $2.text+$3.text;$$.dependsOn = $1.dependsOn + $3.dependsOn;}
        | relational_expression '>' shift_expression
          {$$.link = $1.link;$$.text = $1.text + $2.text+$3.text;$$.dependsOn = $1.dependsOn + $3.dependsOn;}
        | relational_expression LE shift_expression
          {$$.link = $1.link;$$.text = $1.text + $2.text+$3.text;$$.dependsOn = $1.dependsOn + $3.dependsOn;}
        | relational_expression GE shift_expression
          {$$.link = $1.link;$$.text = $1.text + $2.text+$3.text;$$.dependsOn = $1.dependsOn + $3.dependsOn;}
        ;

equality_expression:
        relational_expression
          {$$.link = $1.link;$$.text = $1.text;$$.dependsOn = $1.dependsOn;}
        | equality_expression EQ relational_expression
          {$$.link = $1.link;$$.text = $1.text + $2.text+$3.text;$$.dependsOn = $1.dependsOn + $3.dependsOn;}
        | equality_expression NE relational_expression
          {$$.link = $1.link;$$.text = $1.text + $2.text+$3.text;$$.dependsOn = $1.dependsOn + $3.dependsOn;}
        ;

AND_expression:
        equality_expression
          {$$.link = $1.link;$$.text = $1.text;$$.dependsOn = $1.dependsOn;}
        | AND_expression '&' equality_expression
          {$$.link = $1.link;$$.text = $1.text + $2.text+$3.text;$$.dependsOn = $1.dependsOn + $3.dependsOn;}
        ;

exclusive_OR_expression:
        AND_expression
          {$$.link = $1.link;$$.text = $1.text;$$.dependsOn = $1.dependsOn;}
        | exclusive_OR_expression '^' AND_expression
          {$$.link = $1.link;$$.text = $1.text + $2.text+$3.text;$$.dependsOn = $1.dependsOn + $3.dependsOn;}
        ;

inclusive_OR_expression:
        exclusive_OR_expression
          {$$.link = $1.link;$$.text = $1.text;$$.dependsOn = $1.dependsOn;}
        | inclusive_OR_expression '|' exclusive_OR_expression
          {$$.link = $1.link;$$.text = $1.text + $2.text+$3.text;$$.dependsOn = $1.dependsOn + $3.dependsOn;}
        ;

logical_AND_expression:
        inclusive_OR_expression
        {$$.link = $1.link;$$.text = $1.text;$$.dependsOn = $1.dependsOn;}
        | logical_AND_expression ANDAND inclusive_OR_expression
        {$$.link = $1.link;$$.text = $1.text + $2.text+$3.text;$$.dependsOn = $1.dependsOn + $3.dependsOn;}
        ;

logical_OR_expression:
        logical_AND_expression
        {$$.link = $1.link;$$.text = $1.text;$$.dependsOn = $1.dependsOn;}
        | logical_OR_expression OROR logical_AND_expression
        {$$.link = $1.link;$$.text = $1.text + $2.text+$3.text;$$.dependsOn = $1.dependsOn + $3.dependsOn;}
        ;


conditional_expression:
        logical_OR_expression
		  {$$.link = $1.link;$$.text = $1.text;$$.dependsOn = $1.dependsOn;}
        ;
 constant_expression:
        conditional_expression
         {$$.link = $1.link;$$.text = $1.text;$$.dependsOn = $1.dependsOn;}
        ;
comma_expression_opt:
        /* Nothing */
        {$$.link = NULL;$$.text = "";}
        | comma_expression
        {$$.link = $1.link;$$.text = $1.text;$$.dependsOn = $1.dependsOn;}
        ;
 comma_expression:
        assignment_expression
         {$$.link = $1.link;$$.text = $1.text;$$.dependsOn = $1.dependsOn;}
         ;
/*///////////Declarations and data model construction/////////////////////////////////////*/

declaration_list:
		declaration
		{$$.link = $1.link;$$.text = $1.text;$$.dependsOn = $1.dependsOn;}
        | declaration_list declaration
        {
			$$.link = new common::SyntaxTreeNode(common::SyntaxTreeNode::NTYPE_SEMICOLON,"",context->lineNumber);
			$$.link->_left = $1.link;
			$$.link->_right = $2.link;
			$$.link->dependsOn = $1.link->dependsOn + $2.link->dependsOn;
			
			$$.text.append("$").append($2.text);//        ,  $
        }
        ;

declaration:
     declaring_list ';'
        {
			$$.link = $1.link;
			$$.text = $1.text;
			$$.dependsOn = $1.dependsOn;
        }
        ;

declaring_list:
        type_specifier identifier_declarator initializer_opt 
		{
		     if ($3.text.length() > 0) {
			    $$.link = new common::SyntaxTreeNode(common::SyntaxTreeNode::NTYPE_VARIABLE_DECLARATION, $1.text + " " + $2.text + " = " + $3.text, $2.lineNumber, $1.lineNumber, $3.lineNumber);
			 } else {
			    $$.link = new common::SyntaxTreeNode(common::SyntaxTreeNode::NTYPE_VARIABLE_DECLARATION, $1.text + " " + $2.text, $2.lineNumber, $1.lineNumber, $2.lineNumber);
			 }
			 $$.text = $2.text; //        .             function_definition
			 $$.dependsOn.push_back($1.text + $2.text);
		}
        ;



type_specifier:
        basic_type_name                 /* Arithmetic or void */
		{$$.link = $1.link;$$.text = $1.text;}
        ;

        
identifier_declarator:
        unary_identifier_declarator
         {
            $$.link = $1.link;
            $$.text = $1.text;
            $$.dependsOn = $1.dependsOn;
          }
        | paren_identifier_declarator
         {$$.link = $1.link;$$.text = $1.text;$$.dependsOn = $1.dependsOn;}
        ;
unary_identifier_declarator:
        postfix_identifier_declarator
         {$$.link = $1.link;$$.text = $1.text; $$.dependsOn = $1.dependsOn;}
        | '*' identifier_declarator
         {$$.link = $1.link;$$.text = $1.text+$2.text; $$.dependsOn = $1.dependsOn;}
     
        ;

postfix_identifier_declarator:
        paren_identifier_declarator postfixing_abstract_declarator
        {
            $$.link = $1.link;
            $$.text = $1.text+$2.text;
            $$.dependsOn = $1.dependsOn + $2.dependsOn;
           
        }
        | '(' unary_identifier_declarator ')'
         {$$.link = $1.link;$$.text = $1.text+$2.text+$3.text;$$.dependsOn = $2.dependsOn;}
        | '(' unary_identifier_declarator ')' postfixing_abstract_declarator
        {$$.link = $1.link;$$.text = $1.text+$2.text+$3.text+$4.text;$$.dependsOn = $2.dependsOn + $4.dependsOn;}
        ;

paren_identifier_declarator:
        IDENTIFIER
        {$$.link = $1.link;$$.text = "data->"+$1.text; $$.dependsOn.push_back($$.text);}
        | '(' paren_identifier_declarator ')'
        {$$.link = $1.link;$$.text = '('+$1.text+')';}
        ;
 postfixing_abstract_declarator:
        array_abstract_declarator
        {$$.link = $1.link;$$.text = $1.text;}
        |'(' ')'
        {
			$$.link = NULL; $$.text = "()";
        }
        |'(' parameter_type_list ')'
        {
			$$.link = $2.link; $$.text = "("; $$.text.append($2.text).append(")"); $$.dependsOn = $2.dependsOn;
        }

parameter_type_list:
        parameter_list
		{$$.link = $1.link;$$.text = $1.text;}
        | parameter_list ',' ELLIPSIS
		{/*not yet supproted*/}
        ;
parameter_declaration:
       type_specifier identifier_declarator
	   {$$.link = $1.link;$$.text = $1.text + " " + $2.text; $$.dependsOn.push_back($1.text + $2.text);}
        ;

parameter_list:
        parameter_declaration
		{$$.link = $1.link;$$.text = $1.text;$$.dependsOn = $1.dependsOn;}
        | parameter_list ',' parameter_declaration
		{$$.link = $1.link;$$.text = $1.text + ',' + $3.text;$$.dependsOn = $1.dependsOn + $3.dependsOn;}
        ;

array_abstract_declarator:
        '[' ']'
        {$$.link = NULL;$$.text = "[]";}
        | '[' constant_expression ']'
        {$$.link = $2.link;$$.text = '['+$2.text+']'; $$.dependsOn = $2.dependsOn;}
        | array_abstract_declarator '[' constant_expression ']'
         {$$.link = $1.link;$$.text = $1.text+$2.text+$3.text+$4.text;$$.dependsOn = $1.dependsOn + $3.dependsOn;}
        ;
        
        
initializer_opt:
		'=' initializer
		{
			$$.text = $2.text;
			$$.dependsOn = $2.dependsOn;
			$$.link = NULL;
		}
		| /*empty*/
		{$$.text = "";$$.link = NULL;}
		;
initializer:   //TODO
        '{' initializer_list '}'
        {
			$$.text = '{'+$2.text+'}';
			$$.link = NULL;
		}
        | '{' initializer_list ',' '}'
        {
			$$.text = $2.text;
			$$.link = NULL;
		}
        | assignment_expression
        {
			$$.text = $1.text;
			$$.link = NULL;
		}
        ;

initializer_list:
        initializer
         {
			$$.text = $1.text;
			$$.link = NULL;
			$$.dependsOn = $1.dependsOn;
		}
        | initializer_list ',' initializer
         {
			$$.text = $1.text+','+$3.text;
			$$.link = NULL;
			$$.dependsOn = $1.dependsOn + $3.dependsOn;
		}
        ;


basic_type_name:
        INT
        {$$.link = NULL; $$.text = $1.text;}
        | CHAR
        {$$.link = NULL; $$.text = $1.text;}
        | SHORT
        {$$.link = NULL; $$.text = $1.text;}
        | LONG
        {$$.link = NULL; $$.text = $1.text;}
        | FLOAT
        {$$.link = NULL; $$.text = $1.text;}
        | DOUBLE
        {$$.link = NULL; $$.text = $1.text;}
        | VOID
        {$$.link = NULL; $$.text = $1.text;}
        | BOOL
        {$$.link = NULL; $$.text = $1.text;}
        ;
		
%%

	

int parse(const char* filename, parser::ParserContext* pContext)
{
	context = pContext;
	context->lineNumber = 1;
	yyin = fopen(filename,"r");
	if(yyin==NULL)
	{
		Log::getInstance()->logMessage(Log::MSG_ERROR, "Parser", "Cannot find file");
		return -1;
	}
	
	#ifdef AUTO_DEBUG


		yydebug = 1;
		std::cout<<"\n";
		std::cout<<"Debug mode enabled. To disable debug mode undefine AUTO_DEBUG. \n";
		std::cout<<"===========Contents of input file================= \n";
		
		std::ifstream f(filename);
		char ch;
		while(f.get(ch))std::cout.put(ch);
		
		std::cout<<"\n";
		std::cout<<"===========End of input file=======================\n";
		
		
	#endif
	return yyparse();
}