#include <stdio.h>
#include <string>

#include <autolog/autoLog.h>
#include <Parser/Parser.h>
#include <Syntax2Automata/Syntax2Automata.h>
#include <syntax2automata/SyntaxHelpers.h>
#include <common/SyntaxTree.h>
#include <common/comm.h>

#define DOESNT_ADD_STATE stateNumber--;
#define UPDATE_LAST_STATE lastNodeType = node->_nodeType;
#define INC_PUSH_ASSUMED _directStates[LAST_ELEMENT]->pushesAssumed++;
#define LAST_ELEMENT _directStates.size() - 1

using namespace s2a;
using namespace common;

#define max(a, b) (a) > (b)? (a): (b);

namespace {




	std::string extractVariableName(const std::string& s)
	{
		return s;
	}

	std::string convertArrayInitializationString(const std::string& initString)
	{
		std::string output;

		std::string name = extractVariableName(initString);
		int firstBracket = initString.find('{');
		if (firstBracket == std::string::npos)
		{
			char temp[255];
			strcpy(temp, "<ArrConv> Wrong string for array initialization: ");
			strcat(temp, initString.c_str());
			throw temp;
		}
		firstBracket++;

		int lastBracket = initString.rfind('}');
		if (lastBracket == std::string::npos)
		{
			char temp[255];
			strcpy(temp, "<ArrConv> Wrong string for array initialization: ");
			strcat(temp, initString.c_str());
			throw temp;
		}
		//lastBracket--;

		std::string init = initString.substr(firstBracket, lastBracket-firstBracket);
		std::string instance;

		char initInstance[255];

		bool finish = false;

		for (int i = 0; !finish; i++)
		{
			int comma = init.find(',');
			if (comma == std::string::npos)
			{
				finish = true;
				comma = init.length();
			}
			instance = init.substr(0, comma);
			init.erase(0, comma+1);
			if(i != 0)
				sprintf(initInstance, "\t\t\t%s[%d] = %s;\n", name.c_str(), i, instance.c_str());
			else
				sprintf(initInstance, "%s[%d] = %s;\n", name.c_str(), i, instance.c_str());
			output.append(initInstance);
		}

		return output;
	}


	void generateAutomataSystemImpl(common::SyntaxTreeNode* node, parser::ParserContext* context, common::AutomataSystem* system)
	{
		if (node == NULL)
		{
			autoLog(Log::MSG_ERROR, "Syntax2Automata", "Syntax tree node is NULL");
			return;
		}

		switch(node->_nodeType)
		{
		case common::SyntaxTreeNode::NTYPE_VARIABLE_DECLARATION:
            if (!system->globalDataModel) {
                system->globalDataModel = new DataModel();
                system->globalDataModel->name = "Global";
            }
            s2a::removeDataPrefix(node->_text);
			system->globalDataModel->variables.push_back(common::Variable(node->_text));
			break;
		case common::SyntaxTreeNode::NTYPE_SEMICOLON:
			generateAutomataSystemImpl(node->_left, context, system);
			generateAutomataSystemImpl(node->_right, context, system);
			break;
		case common::SyntaxTreeNode::NTYPE_PROCEDURE:
			{
				AutomatonConverter a;
				a.process(node, false); 
				system->automata.push_back(a.getAutomaton(node));
			}
			break;
		default:
			autoLog(Log::MSG_WARNING, "Syntax2Automata", "Unknown type of syntax tree node");
			return;

		}

	}

	

}



///////////////////////////////////////////////////////////////////////////////////////////////////////
//
//	class AutomatonConverter 
//
///////////////////////////////////////////////////////////////////////////////////////////////////////

int AutomatonConverter::countChildren(const common::SyntaxTreeNode* node, TraverseType mode)
{
	common::SyntaxTreeNode::NodeType previousNodeType = common::SyntaxTreeNode::NTYPE_SOMETHING;
	int result = 0;
	if (mode == TRAVERSE_BOTH_CHILDREN || mode == TRAVERSE_LEFT_CHILD)
	{
		result += countChildrenImpl(node->_left, previousNodeType);
	}
	if (mode == TRAVERSE_BOTH_CHILDREN || mode == TRAVERSE_RIGHT_CHILD)
	{
		result += countChildrenImpl(node->_right, previousNodeType);
	}
	return result;
}

void AutomatonConverter::addStates()
{

	_directStates.push_back(new common::AutomatonState(stateNumber));
	_reverseStates.push_back(new common::AutomatonState(stateNumber));
}

void AutomatonConverter::addDefaultTransitions()
{
	common::Transition* t = new Transition(stateNumber + 1, "", false);
	_directStates[LAST_ELEMENT]->transitions.push_back(t);
	common::Transition* reverseT = new Transition(stateNumber - 1, "", false);
	_reverseStates[LAST_ELEMENT]->transitions.push_back(reverseT);
}

void AutomatonConverter::assignLineNumbers(const common::SyntaxTreeNode* node)
{
	_directStates[LAST_ELEMENT]->firstSourceString = max(node->_StringNumber, _directStates[LAST_ELEMENT]->firstSourceString);
	_directStates[LAST_ELEMENT]->lastSourceString = max(node->_StringNumber, _directStates[LAST_ELEMENT]->lastSourceString);
}

int AutomatonConverter::countChildrenImpl(const common::SyntaxTreeNode *node, common::SyntaxTreeNode::NodeType& previousNodeType)//TODO:      
{
	
	if(node == NULL) return 0;

	int result = 0;

	switch (node->_nodeType)
	{
	case common::SyntaxTreeNode::NTYPE_OPERATOR: case common::SyntaxTreeNode::NTYPE_VARIABLE_DECLARATION:
		if (previousNodeType != common::SyntaxTreeNode::NTYPE_OPERATOR && previousNodeType != common::SyntaxTreeNode::NTYPE_VARIABLE_DECLARATION)
		{
			result++;
			previousNodeType = common::SyntaxTreeNode::NTYPE_OPERATOR;
		}
		break;

	case common::SyntaxTreeNode::NTYPE_IF:
		if (node->_right == NULL)
		{
			result += IF_STATES;
		}else
		{
 			result += IF_WITH_ELSE_STATES;
		}


		previousNodeType = common::SyntaxTreeNode::NTYPE_IF;
		break;

	case common::SyntaxTreeNode::NTYPE_WHILE:
		result += WHILE_STATES;
		previousNodeType = common::SyntaxTreeNode::NTYPE_WHILE;
		break;

	case SyntaxTreeNode::NTYPE_SEMICOLON:
		break;
	case SyntaxTreeNode::NTYPE_RETURN:
		result++;
		break;
	case SyntaxTreeNode::NTYPE_UNKNOWN_FUNCTION_CALL:
	case SyntaxTreeNode::NTYPE_KNOWN_FUNCTION_CALL:
		result++;
		break;
	default:
		autoLog(Log::MSG_ERROR, "Syntax2Automata", "Unknown type for tree node");
	}
	result += countChildrenImpl(node->_left, previousNodeType);
    if (node->_nodeType == SyntaxTreeNode::NTYPE_IF) {
        previousNodeType = SyntaxTreeNode::NTYPE_IF;
    }
	result += countChildrenImpl(node->_right, previousNodeType);
	return result;
}





void AutomatonConverter::process(common::SyntaxTreeNode* node, bool hasGlobalData)
{
	
	if (node == NULL)
	{
		return;
	}
// 	if (node->_nameOfVariableToSave != "")
// 	{
// 		//node->_nameOfVariableToSave = s2a::converToGlobalName(node->_nameOfVariableToSave);
// 	}
   
	stateNumber++;
	switch (node->_nodeType)					// 
	{
		
	case common::SyntaxTreeNode::NTYPE_PROCEDURE:	
		{
			
		DOESNT_ADD_STATE				
			
        s2a::cleanString(node->_text);
        common::Function func(node->_text); // -

        automatonName = func.name;
        _parameters = func.parameters;
        dataModel.variables = func.parameters;
       
		UPDATE_LAST_STATE
		process(node->_left, hasGlobalData);

		break;
		}

		//  
		//    ,      
		//    
		//   -     
	case common::SyntaxTreeNode::NTYPE_VARIABLE_DECLARATION:
		{
			common::Variable* var = new common::Variable(node->_text);
			dataModel.variables.push_back(*var);

            dataModel.cleanVariableNames();

			int initBeginPos = node->_text.find("=");
			if (initBeginPos == std::string::npos)
			{
				DOESNT_ADD_STATE
				break;
			}

			std::string init = node->_text.substr(initBeginPos + 1, node->_text.length() - initBeginPos);

			if (init.length() == 0)
			{
				DOESNT_ADD_STATE
				break;
				
			}
			std::string fullStringWithoutSeparators = var->getName() + " = " + init;
			if( init.find('{') !=  std::string::npos) {//      { }
				init = ::convertArrayInitializationString(fullStringWithoutSeparators);
			} else {
				init = var->getName() + " =" + init;
			}
			std::string name = var->getName();
			if(lastNodeType == common::SyntaxTreeNode::NTYPE_OPERATOR)
			{
				DOESNT_ADD_STATE
				INC_PUSH_ASSUMED
				_directStates[LAST_ELEMENT]->actions.push_back("stack_->push(&" + name + ", sizeof(" + name + "))");	// 
				_directStates[LAST_ELEMENT]->actions.push_back(init);					// 
				_reverseStates[LAST_ELEMENT]->actions.push_back("stack_->pop(&" + name + ")");	//     
				_directStates[LAST_ELEMENT]->lastSourceString = max(node->_StringNumber, _directStates[LAST_ELEMENT]->lastSourceString);
                for (unsigned int i = 0; i < node->dependsOn.size(); i++) {
                    _directStates[LAST_ELEMENT]->stateData->dependsOn.push_back(node->dependsOn[i]);
                }

                _directStates[LAST_ELEMENT]->stateData->changeSet.push_back(name);
			}
			else
			{
				
				addStates();

                _directStates[LAST_ELEMENT]->stateData = new common::AutomatonStateData(AutomatonStateData::STATE_TYPE_OPERATOR);
                _directStates[LAST_ELEMENT]->stateData->dependsOn = node->dependsOn;
                _directStates[LAST_ELEMENT]->stateData->changeSet.push_back(name);

       

				INC_PUSH_ASSUMED

				_directStates[LAST_ELEMENT]->actions.push_back("stack_->push(&" + name + ", sizeof(" + name + "))");	// 
				_directStates[LAST_ELEMENT]->actions.push_back(init);					// 
				_reverseStates[LAST_ELEMENT]->actions.push_back("stack_->pop(&"   +   name   +   ")");	//     
				_directStates[LAST_ELEMENT]->lastSourceString = node->_StringNumber;

				addDefaultTransitions();

				_directStates[LAST_ELEMENT]->firstSourceString = node->_StringNumber;
				_directStates[LAST_ELEMENT]->lastSourceString = node->_StringNumber;
			}

			lastNodeType = common::SyntaxTreeNode::NTYPE_OPERATOR; //    ,       
		}
		break;

	case common::SyntaxTreeNode::NTYPE_OPERATOR:				//
		//node->_text = s2a::converToGlobalName(node->_text); 
        s2a::extractGlobalNames(node->_text, &dataModel);
        s2a::extractGlobalNames(node->_nameOfVariableToSave, &dataModel);

		if(lastNodeType == common::SyntaxTreeNode::NTYPE_OPERATOR)//     =>     
		{
			DOESNT_ADD_STATE
			INC_PUSH_ASSUMED
			if(node->_nameOfVariableToSave != "") {
				_directStates[LAST_ELEMENT]->actions.push_back("stack_->push(&"+node->_nameOfVariableToSave+", sizeof("+node->_nameOfVariableToSave+"))");	// 
			}
			_directStates[LAST_ELEMENT]->actions.push_back(node->_text);					// 
			if(node->_nameOfVariableToSave != "") {
				_reverseStates[LAST_ELEMENT]->actions.push_back("stack_->pop(&"   +   node->_nameOfVariableToSave   +   ")");	//     
			}
			_directStates[LAST_ELEMENT]->lastSourceString = max(node->_StringNumber, _directStates[LAST_ELEMENT]->lastSourceString);
            
            for (unsigned int i = 0; i < node->dependsOn.size(); i++) {
                _directStates[LAST_ELEMENT]->stateData->dependsOn.push_back(node->dependsOn[i]);
            }
            
            _directStates[LAST_ELEMENT]->stateData->changeSet.push_back(node->_nameOfVariableToSave);
            

		}

		else
		{
										//  
			addStates();
			addDefaultTransitions();
            _directStates[LAST_ELEMENT]->stateData = new common::AutomatonStateData(AutomatonStateData::STATE_TYPE_OPERATOR);
            _directStates[LAST_ELEMENT]->stateData->dependsOn = node->dependsOn;
            _directStates[LAST_ELEMENT]->stateData->changeSet.push_back(node->_nameOfVariableToSave);

			INC_PUSH_ASSUMED

			if(node->_nameOfVariableToSave != "") {
				_directStates[LAST_ELEMENT]->actions.push_back("stack_->push(&"+node->_nameOfVariableToSave+", sizeof("+node->_nameOfVariableToSave+"))");
				_reverseStates[LAST_ELEMENT]->actions.push_back("stack_->pop(&"   +   node->_nameOfVariableToSave   +   ")");
			}
			_directStates[LAST_ELEMENT]->actions.push_back(node->_text);
				
			
			_directStates[LAST_ELEMENT]->firstSourceString = node->_StringNumber;
			_directStates[LAST_ELEMENT]->lastSourceString = node->_StringNumber;

		}
		UPDATE_LAST_STATE		//   
		break;

	case common::SyntaxTreeNode::NTYPE_SEMICOLON:			//

		DOESNT_ADD_STATE
		process(node->_left, hasGlobalData);	//  
		process(node->_right, hasGlobalData);
		break;

	case common::SyntaxTreeNode::NTYPE_RETURN:				//

		//node->_text = converToGlobalName(node->_text);
        s2a::extractGlobalNames(node->_text, &dataModel);
        s2a::extractGlobalNames(node->_nameOfVariableToSave, &dataModel);
		addStates();
        _directStates[LAST_ELEMENT]->stateData = new common::AutomatonStateData(AutomatonStateData::STATE_TYPE_OPERATOR);
        _directStates[LAST_ELEMENT]->stateData->dependsOn = node->dependsOn;

		_directStates[LAST_ELEMENT]->actions.push_back(node->_text+"//This is return state. It stores return _directStates in stack(if any _directStates exists)(not yet supported) and switches to last state.");
		//     ,     generateReturnStates()
		
		_indexesOfReturnStates.push_back(LAST_ELEMENT);	//       
	
		_reverseStates[LAST_ELEMENT]->transitions.push_back( new Transition(stateNumber-1,"",false));

		_directStates[LAST_ELEMENT]->firstSourceString = node->_StringNumber;
		_directStates[LAST_ELEMENT]->lastSourceString = node->_StringNumber;
	    UPDATE_LAST_STATE
			break;

	case common::SyntaxTreeNode::NTYPE_IF:  
		{


            s2a::extractGlobalNames(node->_text, &dataModel);
            s2a::extractGlobalNames(node->_nameOfVariableToSave, &dataModel);
			
		   addStates();
           _directStates[LAST_ELEMENT]->stateData = new common::AutomatonStateData(AutomatonStateData::STATE_TYPE_CONDITIONAL);
           _directStates[LAST_ELEMENT]->stateData->dependsOn = node->dependsOn;

			_directStates[LAST_ELEMENT]->transitions.push_back(new Transition(stateNumber+1,node->_text,true));
			Transition* ifFalse;
			int ifStateNumber = stateNumber;

			if (node->_right != NULL) {//       ,      
                ifFalse = new Transition(stateNumber + countChildren(node) + 2, "!(" + node->_text + ')', true/*"stack_->push(false)"*/);				
                _directStates[LAST_ELEMENT]->stateData->subordinateGroups = 2;
                _directStates[LAST_ELEMENT]->stateData->subordinateStates[0] = ifFalse->to;
                _directStates[LAST_ELEMENT]->stateData->subordinateStates[1] = countChildren(node,TRAVERSE_RIGHT_CHILD) + 2;
			} else {
                ifFalse = new Transition(stateNumber + countChildren(node,TRAVERSE_LEFT_CHILD) + 1,"!(" + node->_text + ')', true, "stack_->push(false)");
                _directStates[LAST_ELEMENT]->stateData->subordinateGroups = 1;
                _directStates[LAST_ELEMENT]->stateData->subordinateStates[0] = ifFalse->to;
			}
			_directStates[LAST_ELEMENT]->transitions.push_back(ifFalse);

			assignLineNumbers(node);


			_reverseStates[LAST_ELEMENT]->transitions.push_back(new Transition(stateNumber-1,"",false));
			int skippedStatesNumber = countChildren(node,TRAVERSE_LEFT_CHILD)+1;


			UPDATE_LAST_STATE
			process(node->_left, hasGlobalData);						// true-

			stateNumber++;
			int oneMoreTempStateNumber = stateNumber;

			if(node->_right == NULL) {
				for (unsigned int i = 0; i < _directStates[LAST_ELEMENT]->transitions.size(); i++) {
					if (_directStates[LAST_ELEMENT]->transitions[i]->to == stateNumber) {
						_directStates[LAST_ELEMENT]->transitions[i]->_action.push_back("stack_->push(true)");
					}
				}


				addStates();			//  
                _directStates[LAST_ELEMENT]->stateData = new common::AutomatonStateData(AutomatonStateData::STATE_TYPE_INTERNAL);
	
				_directStates[LAST_ELEMENT]->transitions.push_back(new Transition(stateNumber+1,"",false));
				
				_directStates[LAST_ELEMENT]->firstSourceString = max(node->lineEnd, _directStates[LAST_ELEMENT]->firstSourceString);
				_directStates[LAST_ELEMENT]->lastSourceString = max(node->lineEnd, _directStates[LAST_ELEMENT]->lastSourceString);

				_reverseStates[LAST_ELEMENT]->transitions.push_back(new Transition (stateNumber-1,"",false));
				_reverseStates[LAST_ELEMENT]->transitions.push_back(new Transition (stateNumber-skippedStatesNumber,"!(stack_->popBool())",true,"break"));

				UPDATE_LAST_STATE
			} else	{
				addStates();
				_directStates[LAST_ELEMENT]->firstSourceString = max(node->lineStart, _directStates[LAST_ELEMENT]->firstSourceString);
				_directStates[LAST_ELEMENT]->lastSourceString = max(node->lineStart, _directStates[LAST_ELEMENT]->lastSourceString);

                _directStates[LAST_ELEMENT]->stateData = new common::AutomatonStateData(AutomatonStateData::STATE_TYPE_INTERNAL);
	
				

				_directStates[LAST_ELEMENT]->transitions.push_back(new Transition(stateNumber+countChildren(node,TRAVERSE_RIGHT_CHILD) + 2,"",false));
				_reverseStates[LAST_ELEMENT]->transitions.push_back(new Transition(stateNumber - 1, "", false));
				UPDATE_LAST_STATE	

				stateNumber++;
				addStates();

				_directStates[LAST_ELEMENT]->transitions.push_back(new Transition(stateNumber+ 1,"",false));
				_reverseStates[LAST_ELEMENT]->transitions.push_back(new Transition(ifStateNumber, "", false));
				_directStates[LAST_ELEMENT]->priority = common::AutomatonState::PRIORITY_NO_STOP;
                _directStates[LAST_ELEMENT]->stateData = new common::AutomatonStateData(AutomatonStateData::STATE_TYPE_INTERNAL);
				UPDATE_LAST_STATE
				process(node->_right, hasGlobalData);
				stateNumber++;
				for (unsigned int i = 0; i < _directStates[LAST_ELEMENT]->transitions.size(); i++)
				{
					if (_directStates[LAST_ELEMENT]->transitions[i]->to == stateNumber)

						_directStates[LAST_ELEMENT]->transitions[i]->_action.push_back("stack_->push(false)");      
				}

				addStates();
				_directStates[LAST_ELEMENT]->firstSourceString = max(node->lineEnd, _directStates[LAST_ELEMENT]->firstSourceString);
				_directStates[LAST_ELEMENT]->lastSourceString = max(node->lineEnd, _directStates[LAST_ELEMENT]->lastSourceString);
                _directStates[LAST_ELEMENT]->stateData = new common::AutomatonStateData(AutomatonStateData::STATE_TYPE_INTERNAL);
				
				_directStates[LAST_ELEMENT]->priority = common::AutomatonState::PRIORITY_NO_STOP;
				_reverseStates[LAST_ELEMENT]->priority = common::AutomatonState::PRIORITY_NO_STOP;
											
				_reverseStates[LAST_ELEMENT]->transitions.push_back(new Transition(oneMoreTempStateNumber,"stack_->popBool()",true, "break;"));
				_reverseStates[LAST_ELEMENT]->transitions.push_back(new Transition(stateNumber - 1,"",false));
				_directStates[LAST_ELEMENT]->transitions.push_back(new Transition(stateNumber + 1,"",false));
				UPDATE_LAST_STATE
				
				}

		}
		break;

	case common::SyntaxTreeNode::NTYPE_WHILE: //     
		{
            s2a::extractGlobalNames(node->_text, &dataModel);
            s2a::extractGlobalNames(node->_nameOfVariableToSave, &dataModel);

			int tmp(stateNumber);
			int skippedStatesNumber = countChildren(node, TRAVERSE_BOTH_CHILDREN);

            _directStates.push_back(new AutomatonState(stateNumber, AutomatonState::PRIORITY_NO_STOP));
			_reverseStates.push_back(new AutomatonState(stateNumber, AutomatonState::PRIORITY_NO_STOP));
			addDefaultTransitions();
			_directStates[LAST_ELEMENT]->actions.push_back("stack_->push(false)");
			

			stateNumber++;
			addStates();
            assignLineNumbers(node);
	
			_directStates[LAST_ELEMENT]->transitions.push_back(new Transition(stateNumber + skippedStatesNumber + 2, "!("+node->_text+')', true));
			_directStates[LAST_ELEMENT]->transitions.push_back(new Transition(stateNumber + 1, node->_text, true));

            _directStates[LAST_ELEMENT]->stateData = new common::AutomatonStateData(AutomatonStateData::STATE_TYPE_CONDITIONAL);
            _directStates[LAST_ELEMENT]->stateData->dependsOn = node->dependsOn;
            _directStates[LAST_ELEMENT]->stateData->subordinateGroups = 1;
            _directStates[LAST_ELEMENT]->stateData->subordinateStates[0] = stateNumber + skippedStatesNumber + 2;

			assignLineNumbers(node);
			
			_reverseStates[LAST_ELEMENT]->transitions.push_back(new Transition(stateNumber + skippedStatesNumber + 1,"stack_->popBool()",true,"break"));
			_reverseStates[LAST_ELEMENT]->transitions.push_back(new Transition(stateNumber - 1,"",false));



			UPDATE_LAST_STATE
			process(node->_left, hasGlobalData);
            ++stateNumber;
			_directStates.push_back(new AutomatonState(stateNumber));
			
            _directStates[LAST_ELEMENT]->firstSourceString = node->lineStart;
            _directStates[LAST_ELEMENT]->lastSourceString = node->lineStart;

			_directStates[LAST_ELEMENT]->transitions.push_back(new Transition(tmp + 1,"",false,"stack_->push(true)"));

			_reverseStates.push_back(new AutomatonState(stateNumber));
						
			_reverseStates[LAST_ELEMENT]->transitions.push_back(new Transition(stateNumber - 1,"",false));
		
			stateNumber++;

			_directStates.push_back(new AutomatonState(stateNumber, AutomatonState::PRIORITY_NO_STOP));
            _reverseStates.push_back(new AutomatonState(stateNumber, AutomatonState::PRIORITY_NO_STOP));
			_directStates[LAST_ELEMENT]->transitions.push_back(new Transition(stateNumber + 1,"",false));
            _reverseStates[LAST_ELEMENT]->transitions.push_back(new Transition(tmp + 1,"",false,""));
   	
		    UPDATE_LAST_STATE	
			
		    }
			break;
	case common::SyntaxTreeNode::NTYPE_UNKNOWN_FUNCTION_CALL:
		// ,       
		//      
		//    ,    
		//       
		{
            s2a::extractGlobalNames(node->_text, &dataModel);
            s2a::extractGlobalNames(node->_nameOfVariableToSave, &dataModel);

			_directStates.push_back(new AutomatonState(stateNumber,AutomatonState::PRIORITY_NO_STOP));
            _directStates[LAST_ELEMENT]->stateData = new common::AutomatonStateData(AutomatonStateData::STATE_TYPE_OPERATOR);
             _directStates[LAST_ELEMENT]->stateData->dependsOn = node->dependsOn;
			Transition* stdTrn   = new Transition (stateNumber+1,"",false);
			_directStates[LAST_ELEMENT]->transitions.push_back(stdTrn);
			INC_PUSH_ASSUMED
			_directStates[LAST_ELEMENT]->actions.push_back("data->saveToStack(stack_)");
			if (hasGlobalData)
			{
				_directStates[LAST_ELEMENT]->actions.push_back("globalData->saveToStack(stack_)");
			}

			_directStates[LAST_ELEMENT]->actions.push_back(node->_text+"()");
			//_directStates[LAST_ELEMENT]->actions.push_back("// this.child_ = new("+ node->_text + "Automata)");


			_reverseStates.push_back(new AutomatonState(stateNumber,AutomatonState::PRIORITY_NO_STOP));
			if (hasGlobalData)
			{
				_reverseStates[LAST_ELEMENT]->actions.push_back("globalData->loadFromStack(stack_)");
			}
			_reverseStates[LAST_ELEMENT]->actions.push_back("data->loadFromStack(stack_)");
			Transition* stdTrnRev  = new Transition(stateNumber-1,"",false);
			_reverseStates[LAST_ELEMENT]->transitions.push_back(stdTrnRev);
		}
		_directStates[LAST_ELEMENT]->firstSourceString = node->_StringNumber;
		_directStates[LAST_ELEMENT]->lastSourceString = node->_StringNumber;
		UPDATE_LAST_STATE
		break;
	case common::SyntaxTreeNode::NTYPE_KNOWN_FUNCTION_CALL:
        s2a::extractGlobalNames(node->_text, &dataModel);
        s2a::extractGlobalNames(node->_nameOfVariableToSave, &dataModel);
		//  ,      
		_directStates.push_back(new AutomatonState(stateNumber,AutomatonState::PRIORITY_NO_STOP));
        _directStates[LAST_ELEMENT]->stateData = new common::AutomatonStateData(AutomatonStateData::STATE_TYPE_AUTOMATON_CALL);
          _directStates[LAST_ELEMENT]->stateData->dependsOn = node->dependsOn;

        _directStates[LAST_ELEMENT]->stateData->stringData = "child_ = " + node->_text + "\n child_->stepForward()";
        


		//      -       
		Transition* finishedTrn = new Transition(stateNumber+1,"child_->isAtEnd()",true);
		finishedTrn->_action.push_back("child_ = NULL");
		//    ,     
		Transition* unfinishedTrn = new Transition(stateNumber,"!(child_->isAtEnd())",true);

        

		_directStates[LAST_ELEMENT]->transitions.push_back(unfinishedTrn);
		_directStates[LAST_ELEMENT]->transitions.push_back(finishedTrn);
		_directStates[LAST_ELEMENT]->actions.push_back("if(child_ == NULL) {\n\
															child_ = new "+ node->_text + ";\n\
															_childAutomata[currentState_].push_back(child_);\n\
															child_->toStart();}");
		//this.child = new("+ node->_text + "Automata)");
		_directStates[LAST_ELEMENT]->actions.push_back("child_->stepForward()");


		_reverseStates.push_back(new AutomatonState(stateNumber,AutomatonState::PRIORITY_NO_STOP));

		Transition* unfinishedTrnRev  = new Transition(stateNumber,"!(child_->isAtStart())",true);
		Transition* finishedTrnRev  = new Transition(stateNumber-1,"child_->isAtStart()",true);
		finishedTrnRev->_action.push_back("child_ = NULL");
		_reverseStates[LAST_ELEMENT]->transitions.push_back(unfinishedTrnRev);
		_reverseStates[LAST_ELEMENT]->transitions.push_back(finishedTrnRev);
		_reverseStates[LAST_ELEMENT]->actions.push_back("child_->stepBackward()");
		//_reverseStates[LAST_ELEMENT]->actions.push_back("if(child_ == NULL){child_ = new "+ node->_text + ";child_->toEnd();}");
		_reverseStates[LAST_ELEMENT]->actions.push_back("if(child_ == NULL){ \n\
																			child_ = _childAutomata[currentState_][_childAutomata[currentState_].size() - 1];\n\
																			_childAutomata[currentState_].pop_back(); child_->toEnd();\n\
																			}");


		_directStates[LAST_ELEMENT]->firstSourceString = node->_StringNumber;
		_directStates[LAST_ELEMENT]->lastSourceString = node->_StringNumber;
		UPDATE_LAST_STATE
		break;
	}


}

void AutomatonConverter::generateReturnStates()
{
	string action;
	char temp[255];
	for(unsigned int i = 0; i < _indexesOfReturnStates.size(); i++)	{ 
		sprintf(temp, "stack_->push(%d);", _indexesOfReturnStates[i]);
		_directStates[_indexesOfReturnStates[i]]->transitions.push_back(new common::Transition(stateNumber + 1, "", false));
		_directStates[_indexesOfReturnStates[i]]->transitions[_directStates[_indexesOfReturnStates[i]]->transitions.size() - 1]->_action.push_back(temp);

	}
}


common::Automaton* AutomatonConverter::getAutomaton(const common::SyntaxTreeNode* root)
{
	common::Automaton* result = new common::Automaton(automatonName);
	result->states = new AutomatonState*[_directStates.size() + 2];
	
	result->states[0] = new AutomatonState(0, common::AutomatonState::PRIORITY_NORMAL); // start state
	result->states[0]->transitions.push_back(new common::Transition(1,"", false));

	result->states[0]->firstSourceString = root->lineStart;
	result->states[0]->lastSourceString = root->lineStart;
    result->states[0]->stateData = new common::AutomatonStateData(AutomatonStateData::STATE_TYPE_START_END);

	result->states[_directStates.size() + 1] = new AutomatonState(_directStates.size(), common::AutomatonState::PRIORITY_NORMAL);


	char number[32];
	itoa(_directStates.size(), number, 10);
	
	
	for (unsigned int i = 0; i < _directStates.size(); i++)
	{
		result->states[i + 1] = _directStates[i];
	
		result->states[i+1]->firstSourceString = _directStates[i]->firstSourceString;
		result->states[i+1]->lastSourceString = _directStates[i]->lastSourceString;
	
		
	}

	result->states[_directStates.size() + 1] = new AutomatonState(_directStates.size() + 1, common::AutomatonState::PRIORITY_NORMAL); // start state
	
	for (unsigned int i = 0; i < result->states[_directStates.size()]->transitions.size(); i++) {
		if (result->states[_directStates.size()]->transitions[i]->to == _directStates.size() + 1) {
			result->states[_directStates.size()]->transitions[i]->_action.push_back(std::string("stack_->push(").append(number).append(")"));
		}
	}
	

	
	

	result->reverseStates = new AutomatonState*[_reverseStates.size() + 2];
	result->reverseStates[0] = new AutomatonState(0, common::AutomatonState::PRIORITY_NORMAL); // start state

	
	result->reverseStates[_directStates.size() + 1] = new AutomatonState(_directStates.size() + 1, common::AutomatonState::PRIORITY_NORMAL);
	result->reverseStates[_directStates.size() + 1]->transitions.push_back(new common::Transition("stack_->popInt()","", false));

	result->states[_directStates.size() + 1]->firstSourceString = root->_StringNumber;
	result->states[_directStates.size() + 1]->lastSourceString = root->_StringNumber;
    result->states[_directStates.size() + 1]->stateData = new common::AutomatonStateData(AutomatonStateData::STATE_TYPE_START_END);

	for (unsigned int i = 0; i < _reverseStates.size(); i++)
	{
		result->reverseStates[i + 1] = _reverseStates[i];
	}

	result->data = dataModel;
	result->data.name = automatonName;
	result->data.cleanVariableNames();
	result->endState = _directStates.size() + 2;
	result->parameters = _parameters;
	generateReturnStates();
	return result;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////
//
//	 Syntax2Automata 
//
///////////////////////////////////////////////////////////////////////////////////////////////////////


common::AutomataSystem* s2a::generateAutomataSystem(common::SyntaxTreeNode* node, parser::ParserContext* context)
{
	common::AutomataSystem* result = new common::AutomataSystem;
	generateAutomataSystemImpl(node, context, result);
	return result;
}
