#ifndef Variable_h__
#define Variable_h__



#include <string>
#include <vector>

using std::string;
using std::vector;

namespace common {

class CTypeID
{
	friend bool operator==(const common::CTypeID& tid1, const common::CTypeID& tid2);
public:
	CTypeID();
	CTypeID(const string& s);
	CTypeID(const CTypeID& t);
	~CTypeID();

	void	create(const string& s, string* name = NULL);
	string	findType(const string& s);
	string	toString() const;
	string	toString(const string& name) const;
	unsigned int getDimensions() const { return dimension_; }
	unsigned int getCheToTam(int i) const { return sizes_[i];}

private:
	
	string					baseName_;
	unsigned int			dimension_;
	vector<unsigned int>	sizes_;
	unsigned int			ptrLevel_;
};

class Variable
{
public:
	friend bool operator==(const common::Variable& lhs, const common::Variable& rhs);
	explicit Variable(const string& s);

	const string& getName() const { return _name; }
	const string getType() const { return _type.toString(); }
	const CTypeID& getTypeId() const { return _type;}
	const string& getDefinition() const { return _definition; }
    bool hasInitializer() { return _initializer.length() > 0; }
    const string& getInitializer() { return _initializer;}
	void cleanName();

private:
	CTypeID _type;
	string	_definition;
    string  _initializer;
	string	_name;
};


bool operator==(const CTypeID& tid1, const CTypeID& tid2);

bool isCharForNumber(char c);
bool isCharForVarName(char c);
bool isCharForVarNameStart(char c);



}//namespace common



#endif // Variable_h__