#include <common/Variable.h>

using namespace common;



//////////////////////////////////////////////////////////////////////////
// CVariable
//////////////////////////////////////////////////////////////////////////

Variable::Variable(const string& s)
{
	_definition = s;
	_type.create(s, &_name);

    int idx = s.find("=");
    if (idx != string::npos) {
        _initializer = s.substr(idx + 1, s.length() - idx - 1);
    }
}

void Variable::cleanName()
{
	int pos = _name.find("data->");
	if (pos != std::string::npos) {
		_name.erase(pos, 6);
		_definition.erase(_definition.find("data->"), 6);
	}
}


//////////////////////////////////////////////////////////////////////////
//Constructors & Destructor
//////////////////////////////////////////////////////////////////////////

CTypeID::CTypeID()
{
	baseName_ = "";
	dimension_ = 0;
	ptrLevel_ = 0;
}

CTypeID::CTypeID(const std::string &s)
{
	baseName_ = "";
	dimension_ = 0;
	ptrLevel_ = 0;
	create(s);
}

CTypeID::CTypeID(const CTypeID& t)
{
	baseName_ = t.baseName_;
	dimension_ = t.dimension_;
	ptrLevel_ = t.ptrLevel_;
	sizes_ = t.sizes_;
}

CTypeID::~CTypeID()
{
	
}

//////////////////////////////////////////////////////////////////////////
//Generates an instance of class based on string such as "int *var[44]"
//////////////////////////////////////////////////////////////////////////

void CTypeID::create(const string& s, string* name)
{
	if (name != NULL)
		name->clear();

	baseName_ = "";
	sizes_.clear();

	//Index for string parser
	int index = -1;
	//State for parser
	int state = 0;

	char c;

	//First, performing "unsigned" check
	if (s.length() > 9)
	{
		if (s.substr(0, 9) == "unsigned ")
		{
			baseName_.append("unsigned ");
			index = 8;
		}
	}

ctypeid_create_auto_start:
	index++;	//Advancing to the next char
	if (index == s.length())
		goto ctypeid_create_auto_end;

	c = s[index];

    if (c == '=') {
		goto ctypeid_create_auto_end;
    }

	switch (state)
	{
	case -1:	//Error
		char temp[255];
		strcpy(temp, "<CTypeID> Error on parsing string: \"");
		strcat(temp, s.c_str());
		strcat(temp, "\"");
		throw temp;

	case 0:		//Start state
		if (c == ' ' || c == '\t'){
			state = 0; //skip whitespaces
			goto ctypeid_create_auto_start;
		}
		if (!isCharForVarNameStart(c))
		{ //not a char for variable name start
			state = -1;
			index--;
			goto ctypeid_create_auto_start;
		}
		else
		{ //OK
			state = 1;
			index--;
			goto ctypeid_create_auto_start;
		}

	case 1:		//State which stores variable type name, just doing append for type name string
		if (c == ' ')
		{
			state = 2;
			goto ctypeid_create_auto_start;
		}
		if (c == '*')
		{
			state = 3;
			goto ctypeid_create_auto_start;
		}
		if (isCharForVarName(c))
		{
			char temp[2];
			temp[0] = c;
			temp[1] = 0;
			baseName_.append(temp);
			state = 1;
			goto ctypeid_create_auto_start;
		}
		else
		{
			state = -1;
			index--;
			goto ctypeid_create_auto_start;
		}

	case 2:		//End of type name
		if (c == '*')
		{
			state = 3;
			goto ctypeid_create_auto_start;
		}
		else
		{
			state = 4;
			index--;
			goto ctypeid_create_auto_start;
		}

	case 3:		//On new *, we increment pointer level
		ptrLevel_++;
		if (c == '*')
		{
			state = 3;
			goto ctypeid_create_auto_start;
		}
		else
		{
			state = 4;
			index--;
			goto ctypeid_create_auto_start;
		}

	case 4:		//Variable name, we store it to output string
		if (c == ' ')
		{
			state = 4;
			goto ctypeid_create_auto_start;
		}
		if (isCharForVarName(c) || c == '-' || c == '>' || c == '.')
		{
			char tmp[2];
			tmp[0] = c;
			tmp[1] = 0;
			if (name != NULL) name->append(tmp);		//append a symbol 
			state = 4;
			goto ctypeid_create_auto_start;
		}
		if (c == '[')
		{
			dimension_++;
			sizes_.push_back(0);
			state = 5;
			goto ctypeid_create_auto_start;
		}
		else
		{
			state = -1;
			index--;
			goto ctypeid_create_auto_start;
		}
	
	case 5:		//State which stores size of one of array's dimensions
		if (c == ']')
		{
			if (sizes_[sizes_.size()-1] == 0)
			{
				state = -1;
				index--;
				goto ctypeid_create_auto_start;
			}
			else
			{
				state = 6;
				goto ctypeid_create_auto_start;
			}
		}
		if (isCharForNumber(c))
		{
			sizes_[sizes_.size()-1] = sizes_[sizes_.size()-1]*10 + (c-0x30);
			state = 5;
			goto ctypeid_create_auto_start;
		}
		else
		{
			state = -1;
			index--;
			goto ctypeid_create_auto_start;
		}

	case 6:
		if (c == ' ')
		{
			state = 6;
			goto ctypeid_create_auto_start;
		}
		if (c == '[')
		{
			dimension_++;
			sizes_.push_back(0);
			state = 5;
			goto ctypeid_create_auto_start;
		}
		else
		{
			state = -1;
			index--;
			goto ctypeid_create_auto_start;
		}
	
	default:
		state = -1;
		index--;
		goto ctypeid_create_auto_start;
	}


ctypeid_create_auto_end:
	if (!(state == 4 || state == 6))
	{
		throw (string("<CTypeID> Error on parsing string: \"").append(s).append("\"").c_str());
	}

}

//////////////////////////////////////////////////////////////////////////
//Returns type for target string, e.g. for "*var[3]" it will be "int"
//////////////////////////////////////////////////////////////////////////

string CTypeID::findType(const std::string &s)
{
	string output = baseName_;
	char temp[2];
	temp[0] = '*';
	temp[1] = 0;
	int depth = dimension_+ptrLevel_;

	bool b = false;

	for (unsigned int i = 0; i < s.length(); i++)
	{
		if (s[i] == '&' && !b)
			depth++;
		if (s[i] == '*' && !b)
			depth--;
		if (s[i] == '[' && !b)   //TODO
			b = true;
		if (s[i] == ']' && b)
		{
			b = false;
			depth--;
		}
	}
	for (int i = 0; i < depth; i++)
		output.append(temp);
	return output;
}

//////////////////////////////////////////////////////////////////////////
//Returns a string which represents current type
//////////////////////////////////////////////////////////////////////////

string CTypeID::toString() const
{
	string s = baseName_;
	char tempDim[8];
	for (unsigned int i = 0; i < ptrLevel_; i++)
	{
		s.append("*");
	}
	for (unsigned int i = 0; i < dimension_; i++)
	{
		sprintf(tempDim, "[%d]", sizes_[i]);
		s.append(tempDim);
	}
	return s;
}

//////////////////////////////////////////////////////////////////////////
//Returns a string which represents current type with a variable name
//////////////////////////////////////////////////////////////////////////

string CTypeID::toString(const string& name) const
{
	string s = baseName_;
	char tempDim[8];
	for (unsigned int i = 0; i < ptrLevel_; i++)
	{
		s.append("*");
	}
	s.append(" ");
	s.append(name);
	for (unsigned int i = 0; i < dimension_; i++)
	{
		sprintf(tempDim, "[%d]", sizes_[i]);
		s.append(tempDim);
	}
	return s;
}

//////////////////////////////////////////////////////////////////////////
//Auxiliary
//////////////////////////////////////////////////////////////////////////
bool common::isCharForNumber(char c)
{
	return (c >= 0x30 && c <= 0x39);
}

bool common::isCharForVarName(char c)
{
	return (isCharForVarNameStart(c)) || (isCharForNumber(c));
}

bool common::isCharForVarNameStart(char c)
{
	return (c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') || (c == '_');
}

bool common::operator==(const common::CTypeID& tid1, const common::CTypeID& tid2)
{
	if (tid1.baseName_ != tid2.baseName_)
		return false;
	if (tid1.dimension_ != tid2.dimension_)
		return false;
	if (tid2.ptrLevel_ != tid2.ptrLevel_)
		return false;

	return true;
}

bool common::operator==(const common::Variable& lhs, const common::Variable& rhs)
{
	return (lhs._definition == rhs._definition && lhs._name == rhs._name && lhs._type == rhs._type);
}