#ifndef ProgramDependencyGraph_h__
#define ProgramDependencyGraph_h__

#include <string>
#include <vector>
#include <fstream>

namespace common {
    struct Automaton;
}

namespace slicing {

struct GraphNode {

    bool isControlFlowStatement;

    int nodeNumber;

    int stateNumber;
    std::string statement;
    
    int lineStart;
    int lineEnd;
    
    std::vector<int> dataDependentEdges;
    std::vector<int> controlDependentEdges;
};

class ProgramDependencyGraph{

    std::vector<GraphNode*> _nodes;

    void buildControlFlowEdges(common::Automaton* automaton);
    void buildDataFlowEdges(common::Automaton* automaton);
  
public:

    static const int magicNumber = 0x000002d90;

    const std::vector<GraphNode*>& getNodes() const {return _nodes;}
    bool create(common::Automaton* automaton);
    ~ProgramDependencyGraph()
    {
        for (unsigned int i = 0; i < _nodes.size(); ++i)
        {
            delete _nodes[i];
        }
    }

};

} // namespace slicing
#endif // ProgramDependencyGraph_h__