#include <stdio.h>
#include <string>

#include "ADBProcessor.h"

ADBProcessor adbProcessor;

const int MAGIC_NUMBER = 0x0ADBF173;

//TODO: some error notification
bool ADBProcessor::loadADBFile(const char* fileName)
{
	FILE* adbFile = fopen(fileName, "rb");
	if (!adbFile)
	{
		printf("Failed to load .adb file \n");
		return false;
	}
	int someValue;
	fread(&someValue, sizeof(int), 1, adbFile); //reading magic number
	if (someValue != MAGIC_NUMBER)	{
		printf("ADB file is damaged or has unknown format. Load failed \n");
		fclose(adbFile);
		return false;
	}
	fread(&someValue, sizeof(int), 1, adbFile); //reading source file name length
	
	char* name = new char[someValue + 1];
	fread(name, sizeof(char), someValue, adbFile);
	name[someValue] = 0;
	_sourceFile = name;
	delete[] name;

	std::string currentAutomaton;
	AutomataStates states;
	int numberOfAutomata;

	fread(&numberOfAutomata, sizeof(int), 1, adbFile); //total number of automata

	for (int i = 0; i < numberOfAutomata; ++i)	{
		fread(&someValue, sizeof(int), 1, adbFile);//length of automaton name
		char* name = new char[someValue + 1];
		fread(name, sizeof(char), someValue, adbFile);//automaton name
		name[someValue] = 0;
		currentAutomaton = name;
		delete[] name;
		int stateNumber;
		fread(&stateNumber, sizeof(int), 1, adbFile);//states in current automaton
		for (int i = 0; i < stateNumber; i++) {
			StateLines lines;
			fread(&lines.lineStart, sizeof(int), 1, adbFile);//length of automaton name
			fread(&lines.lineEnd, sizeof(int), 1, adbFile);//length of automaton name
			lines.lineStart--; 
			lines.lineEnd--;
			states._stateLinesMap[i] = lines;
		}
		_states[currentAutomaton] = states;

        _loadPDG(adbFile, name);
	}
  /*
      for (int i = 0; i < numberOfAutomata; ++i)
        {
            _loadPDG(adbFile, i);
        }*/
	fclose(adbFile);
    
	return true;

}

const char* ADBProcessor::getSourceFileName()
{
	return _sourceFile.c_str();
}


StateLines ADBProcessor::getLinesForState( const std::string& automatonName, int stateNumber )
{
	return _states[automatonName]._stateLinesMap[stateNumber];
}

void ADBProcessor::_loadPDG( FILE* file, std::string name )
{
    PDGraphs.push_back(new PDGraph());
    PDGraphs[PDGraphs.size() - 1]->automatonName = name;
    unsigned int nodes;
    fread(&nodes, sizeof(unsigned int), 1, file);

    for(unsigned int i = 0; i < nodes; ++i)
    {
        PDGNode node;

        fread(&(node.lines.lineStart), sizeof(int), 1, file);
        fread(&(node.lines.lineEnd), sizeof(int), 1, file);
        fread(&(node.stateNumber), sizeof(int), 1, file);

       
        fread(&(node.controlDepsNumber), sizeof(int), 1, file);
        if (node.controlDepsNumber > 0)
        {
            node.controlDependencies = new int[node.controlDepsNumber];
        } else {
            node.controlDependencies = NULL;
        }
        

        for (int j = 0; j < node.controlDepsNumber; ++j)
        {
            fread(&(node.controlDependencies[j]), sizeof(int), 1, file);
        }


        fread(&(node.dataDepsNumber), sizeof(int), 1, file);
        if (node.dataDepsNumber > 0)
        {
            node.dataDependencies = new int[node.dataDepsNumber];
        } else {
            node.dataDependencies = NULL;
        }

        for (int j = 0; j < node.dataDepsNumber; ++j)
        {
            fread(&(node.dataDependencies[j]), sizeof(int), 1, file);
        }
        node.graphNumber = PDGraphs.size() - 1;
        PDGraphs[PDGraphs.size() - 1]->nodes.push_back(node);
    }



}

int ADBProcessor::getSliceForState( PDGNode& node, StateLines*& slice )
{
    std::vector<StateLines> s;
   
   _traverseGraph(node, s);
   

    slice = new StateLines[s.size()];
    for (unsigned int i = 0; i < s.size(); ++i)
    {
        slice[i] = s[i];
        --slice[i];
    }
    return s.size();
}

void ADBProcessor::_traverseGraph( PDGNode& node, std::vector<StateLines>& slice )
{
    for (int i = 0; i < node.controlDepsNumber; ++i)
    {
        _traverseGraph(PDGraphs[node.graphNumber]->nodes[node.controlDependencies[i]], slice);
    }
    for (int i = 0; i < node.dataDepsNumber; ++i)
    {
        _traverseGraph(PDGraphs[node.graphNumber]->nodes[node.dataDependencies[i]], slice);
    }

    slice.push_back(node.lines);
}

PDGNode* ADBProcessor::getStateForLine( int lineNumber )
{
    for (unsigned int i = 0; i < PDGraphs.size(); ++i )
    {
        for (unsigned int j = 0; j < PDGraphs[i]->nodes.size(); ++j)
        {
            if (PDGraphs[i]->nodes[j].lines.isInside(lineNumber))
            {
                return &PDGraphs[i]->nodes[j];
            }
        }
    }
    return NULL;
}